@@ -315,16 +315,16 @@ static void test_read_err_code(void) {
     memset(&err, -1, sizeof(err));
     yyjson_doc_free(yyjson_read_opts((char *)str, strlen(str),
                                      YYJSON_READ_ALLOW_COMMENTS, NULL, &err));
-    yy_assert(err.code == YYJSON_READ_ERROR_INVALID_COMMENT);
-    yy_assert(err.pos == strlen(str) - 2);
+    yy_assert(err.code == YYJSON_READ_ERROR_UNEXPECTED_END);
+    yy_assert(err.pos == strlen(str));
     
     str = "[123/*";
     //         ^ unclosed multiline comment
     memset(&err, -1, sizeof(err));
     yyjson_doc_free(yyjson_read_opts((char *)str, strlen(str),
                                      YYJSON_READ_ALLOW_COMMENTS, NULL, &err));
-    yy_assert(err.code == YYJSON_READ_ERROR_INVALID_COMMENT);
-    yy_assert(err.pos == strlen(str) - 2);
+    yy_assert(err.code == YYJSON_READ_ERROR_UNEXPECTED_END);
+    yy_assert(err.pos == strlen(str));
 #endif
     
     
@@ -370,7 +370,7 @@ static void test_read_err_code(void) {
     memset(&err, -1, sizeof(err));
     yyjson_doc_free(yyjson_read_opts((char *)str, strlen(str), 0, NULL, &err));
     yy_assert(err.code == YYJSON_READ_ERROR_INVALID_STRING);
-    yy_assert(err.pos == 7);
+    yy_assert(err.pos == 1);
     
     // invalid 1-byte UTF-8
     memcpy(buf, "\"abcdefgh\"", 10);
@@ -509,6 +509,25 @@ static void test_read_err_code(void) {
     
     
     
+    // -------------------------------------------------------------------------
+    // UTF-8 BOM
+    buf[0] = 0xEF;
+    buf[1] = 0xBB;
+    buf[2] = 0xBF;
+    memcpy(buf + 3, "abcde", 6);
+    memset(&err, -1, sizeof(err));
+    yyjson_doc_free(yyjson_read_opts((char *)buf, strlen(buf), 0, NULL, &err));
+    yy_assert(err.code == YYJSON_READ_ERROR_UNEXPECTED_CHARACTER);
+    yy_assert(err.pos == 0);
+    
+#if !YYJSON_DISABLE_NON_STANDARD
+    memset(&err, -1, sizeof(err));
+    yyjson_doc_free(yyjson_read_opts((char *)buf, strlen(buf), YYJSON_READ_ALLOW_BOM, NULL, &err));
+    yy_assert(err.code == YYJSON_READ_ERROR_UNEXPECTED_CHARACTER);
+    yy_assert(err.pos == 3);
+#endif
+    
+    
     // -------------------------------------------------------------------------
     // Invalid JSON literal, such as `truu`.
     str = "[truu]";
@@ -636,7 +655,122 @@ static void test_write_err_code(void) {
 #endif
 }
 
+
+
+static void test_locate_pos(void) {
+    const char *str;
+    size_t len, pos, line, col, chr;
+    
+    // -------------------------------------------------------------------------
+    // Invalid input.
+    yy_assert(!yyjson_locate_pos(NULL, 0, 0, NULL, NULL, NULL));
+    
+    line = col = chr = SIZE_MAX;
+    yy_assert(!yyjson_locate_pos(NULL, 0, 0, &line, &col, &chr));
+    yy_assert(line == 0 && col == 0 && chr == 0);
+    
+    yy_assert(!yyjson_locate_pos("abc", 3, 4, NULL, NULL, NULL));
+    
+    line = col = chr = SIZE_MAX;
+    yy_assert(!yyjson_locate_pos("abc", 3, 4, &line, &col, &chr));
+    yy_assert(line == 0 && col == 0 && chr == 0);
+    
+    // -------------------------------------------------------------------------
+    // Empty.
+    yy_assert(yyjson_locate_pos("", 0, 0, &line, &col, &chr));
+    yy_assert(line == 1 && col == 1 && chr == 0);
+    
+    // -------------------------------------------------------------------------
+    // Empty new line.
+    yy_assert(yyjson_locate_pos("\n", 1, 0, &line, &col, &chr));
+    yy_assert(line == 1 && col == 1 && chr == 0);
+    yy_assert(yyjson_locate_pos("\n", 1, 1, &line, &col, &chr));
+    yy_assert(line == 2 && col == 1 && chr == 1);
+    yy_assert(yyjson_locate_pos("\n\n", 2, 1, &line, &col, &chr));
+    yy_assert(line == 2 && col == 1 && chr == 1);
+    yy_assert(yyjson_locate_pos("\n\n", 2, 2, &line, &col, &chr));
+    yy_assert(line == 3 && col == 1 && chr == 2);
+    
+    // -------------------------------------------------------------------------
+    // 1 line.
+    str = "abc";
+    len = strlen(str);
+    for (pos = 0; pos <= len; pos++) {
+        yy_assert(yyjson_locate_pos(str, len, pos, &line, &col, &chr));
+        yy_assert(line == 1 && col == pos + 1 && chr == pos);
+    }
+    
+    // -------------------------------------------------------------------------
+    // 2 lines.
+    str = "abc\ndef";
+    len = strlen(str);
+    for (pos = 0; pos <= len; pos++) {
+        yy_assert(yyjson_locate_pos(str, len, pos, &line, &col, &chr));
+        if (pos <= 3) {
+            yy_assert(line == 1 && col == pos + 1 && chr == pos);
+        } else {
+            yy_assert(line == 2 && col == pos - 4 + 1 && chr == pos);
+        }
+    }
+    
+    // -------------------------------------------------------------------------
+    // 3 lines.
+    str = "abc\ndef\nghijklmn";
+    len = strlen(str);
+    for (pos = 0; pos <= len; pos++) {
+        yy_assert(yyjson_locate_pos(str, len, pos, &line, &col, &chr));
+        if (pos <= 3) {
+            yy_assert(line == 1 && col == pos + 1 && chr == pos);
+        } else if (pos <= 7) {
+            yy_assert(line == 2 && col == pos - 4 + 1 && chr == pos);
+        } else {
+            yy_assert(line == 3 && col == pos - 8 + 1 && chr == pos);
+        }
+    }
+    
+    // -------------------------------------------------------------------------
+    // Unicode.
+    str = "abcé果😀"; // 1-4 byte UTF-8
+    len = strlen(str);
+    for (pos = 0; pos <= len; pos++) {
+        size_t pos_uni = pos;
+        if (4 <= pos && pos <= 5) pos_uni = 4;
+        if (6 <= pos && pos <= 8) pos_uni = 5;
+        if (9 <= pos && pos <= 12) pos_uni = 6;
+        yy_assert(yyjson_locate_pos(str, len, pos, &line, &col, &chr));
+        yy_assert(line == 1 && col == pos_uni + 1 && chr == pos_uni);
+    }
+    str = "abcdef"; // invalid UTF-8
+    len = strlen(str);
+    char buf[7] = { 0 };
+    memcpy(buf, str, len + 1);
+    buf[1] = 0x80;
+    buf[2] = 0xF8;
+    for (pos = 0; pos <= len; pos++) {
+        yy_assert(yyjson_locate_pos(buf, len, pos, &line, &col, &chr));
+        yy_assert(line == 1 && col == pos + 1 && chr == pos);
+    }
+    // UTF-8 BOM.
+    buf[0] = 0xEF;
+    buf[1] = 0xBB;
+    buf[2] = 0xBF;
+    for (pos = 0; pos <= len; pos++) {
+        yy_assert(yyjson_locate_pos(buf, len, pos, &line, &col, &chr));
+        if (pos < 3) {
+            // Don't allow BOM, pos should always be 0.
+            yy_assert(line == 1 && col == (pos ? 2 : 1) && chr == (pos ? 1 : 0));
+        } else {
+            // Allow BOM, don't count BOM as a character.
+            size_t pos_uni = pos - 3;
+            yy_assert(line == 1 && col == pos_uni + 1 && chr == pos_uni);
+        }
+    }
+}
+
+
+
 yy_test_case(test_err_code) {
     test_read_err_code();
     test_write_err_code();
+    test_locate_pos();
 }
