@@ -665,6 +665,19 @@ static void test_json_write(yyjson_alc *alc) {
                         "    ]\n"
                         "}");
     
+    root = yyjson_mut_obj(doc);
+    yyjson_mut_doc_set_root(doc, root);
+    val = yyjson_mut_arr(doc);
+    yyjson_mut_arr_add_val(val, yyjson_mut_raw(doc, "123"));
+    yyjson_mut_obj_add_val(doc, root, "a", val);
+    validate_json_write(doc, alc,
+                        "{\"a\":[123]}",
+                        "{\n"
+                        "    \"a\": [\n"
+                        "        123\n"
+                        "    ]\n"
+                        "}");
+    
     root = yyjson_mut_obj(doc);
     yyjson_mut_doc_set_root(doc, root);
     val = yyjson_mut_obj(doc);
@@ -844,6 +857,16 @@ yy_test_case(test_json_writer) {
         yy_assert(strcmp(ret, "[\"abc\"]") == 0);
         free(ret);
         
+        yyjson_set_str(val, "abc\n");
+        yyjson_set_str_noesc(val, true);
+        ret = yyjson_write(doc, 0, NULL);
+        yy_assert(strcmp(ret, "[\"abc\n\"]") == 0);
+        free(ret);
+        yyjson_set_str_noesc(val, false);
+        ret = yyjson_write(doc, 0, NULL);
+        yy_assert(strcmp(ret, "[\"abc\\n\"]") == 0);
+        free(ret);
+        
         yyjson_set_strn(val, "abcd", 3);
         ret = yyjson_write(doc, 0, NULL);
         yy_assert(strcmp(ret, "[\"abc\"]") == 0);
@@ -873,6 +896,73 @@ yy_test_case(test_json_writer) {
         yyjson_doc_free(doc);
         yyjson_mut_doc_free(mdoc);
     }
+    
+    
+    // test newline at end
+    {
+        size_t len;
+        const char *str;
+        yyjson_doc *doc;
+        yyjson_mut_doc *mdoc;
+        char *ret;
+        
+        // single value
+        str = "123";
+        doc = yyjson_read(str, strlen(str), 0);
+        ret = yyjson_write(doc, YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        mdoc = yyjson_doc_mut_copy(doc, NULL);
+        ret = yyjson_mut_write(mdoc, YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        yyjson_doc_free(doc);
+        yyjson_mut_doc_free(mdoc);
+        
+        // multiple values
+        str = "[123]";
+        doc = yyjson_read(str, strlen(str), 0);
+        ret = yyjson_write(doc, YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        mdoc = yyjson_doc_mut_copy(doc, NULL);
+        ret = yyjson_mut_write(mdoc, YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        yyjson_doc_free(doc);
+        yyjson_mut_doc_free(mdoc);
+        
+        // multiple values, pretty
+        str = "[\n    123\n]";
+        doc = yyjson_read(str, strlen(str), 0);
+        ret = yyjson_write(doc, YYJSON_WRITE_PRETTY | YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        mdoc = yyjson_doc_mut_copy(doc, NULL);
+        ret = yyjson_mut_write(mdoc, YYJSON_WRITE_PRETTY | YYJSON_WRITE_NEWLINE_AT_END, &len);
+        yy_assert(strlen(ret) == len && len == strlen(str) + 1);
+        yy_assert(memcmp(str, ret, strlen(str)) == 0);
+        yy_assert(ret[strlen(str)] == '\n');
+        free(ret);
+        
+        yyjson_doc_free(doc);
+        yyjson_mut_doc_free(mdoc);
+    }
 #endif
     
     // test build JSON on stack
@@ -909,6 +999,43 @@ yy_test_case(test_json_writer) {
         char *json = yyjson_mut_val_write_opts(&root, 0, &alc, NULL, NULL);
         yy_assert(strcmp(json, expect) == 0);
     }
+    
+    // test bool conversion
+    // some environments don't have a native bool type
+    // and the bool type may store values other than 0/1
+    {
+        yyjson_mut_doc *mdoc = yyjson_mut_doc_new(NULL);
+        yyjson_mut_val *mobj = yyjson_mut_obj(mdoc);
+        yyjson_mut_doc_set_root(mdoc, mobj);
+        
+        for (u8 i = 0; i < 10; i++) {
+            char str[2];
+            snprintf(str, sizeof(str), "%d", (int)i);
+            yyjson_mut_val *key = yyjson_mut_strcpy(mdoc, str);
+            yyjson_mut_val *val = yyjson_mut_bool(mdoc, (bool)i);
+            yyjson_mut_obj_add(mobj, key, val);
+        }
+        
+        char *json = yyjson_mut_write(mdoc, YYJSON_WRITE_PRETTY, NULL);
+        yy_assert(json != NULL);
+        
+#if !YYJSON_DISABLE_READER
+        yyjson_doc *doc = yyjson_read(json, strlen(json), 0);
+        yyjson_val *obj = yyjson_doc_get_root(doc);
+        yy_assert(yyjson_obj_size(obj) == 10);
+        for (u8 i = 0; i < 10; i++) {
+            char str[2];
+            snprintf(str, sizeof(str), "%d", (int)i);
+            yyjson_val *val = yyjson_obj_get(obj, str);
+            yy_assert(yyjson_is_bool(val));
+            yy_assert(yyjson_get_bool(val) == (i != 0));
+        }
+        yyjson_doc_free(doc);
+#endif
+        
+        yyjson_mut_doc_free(mdoc);
+        free(json);
+    }
 }
 
 #else
